#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
# Author         : Th.L.
# creation Date  : 2018-01-25
# file name      : cisco-bug.py
#
# add on for cisco support api. calls cisco bug api 2.0.
# for more information see: https://developer.cisco.com/docs/support-apis/#bug
#
#
#
import logging
import ConfigParser
import ciscosupport
import ciscoapi
import os
import json
import time
import sys


def main():

    def get_new_token():
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(time.time()) - int(response.get('lifetime', 3600)) - 30
        return access_token, lifetime

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    base_path = '~/var/ciscoapi'
    conf_file = '~/etc/ciscoapi/ciscoapi.conf'
    conf_file = os.path.expanduser(conf_file)

    access_token = ''
    lifetime = 0

    refresh_found = 2
    refresh_notfound = 1

    wait_after_start = True
    max_wait_time = 15

    loglevel = 'warning'

    if os.path.isfile(conf_file):
        configParser = ConfigParser.RawConfigParser()
        configParser.read(conf_file)
        # global options
        if configParser.has_option('global', 'base_path'):
            base_path = configParser.get('global', 'base_path')
        if configParser.has_option('global', 'wait_after_start'):
            wait_after_start = configParser.getboolean('global', 'wait_after_start')
        if configParser.has_option('global', 'max_wait_time'):
            max_wait_time = configParser.getint('global', 'max_wait_time')
        if configParser.has_option('global', 'loglevel'):
            loglevel = configParser.get('global', 'loglevel')
        # bug api specific options
        if configParser.has_option('bug', 'refresh_found'):
            refresh_found = configParser.getint('bug', 'refresh_found')
        if configParser.has_option('bug', 'refresh_notfound'):
            refresh_notfound = configParser.getint('bug', 'refresh_notfound')
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    # loglevel = 'debug'
    ciscosupport.set_logging(loglevel)

    bug_path = base_path + '/bug'
    path_found = ciscosupport.expand_path(bug_path + '/found')
    path_not_found = ciscosupport.expand_path(bug_path + '/not_found')
    path_request = ciscosupport.expand_path(bug_path + '/request')
    path_missing = ciscosupport.expand_path(bug_path + '/missing')

    pids_requested = {}
    pids_refresh = {}
    pids = []

    # get list of bug reports to refresh
    pids = ciscosupport.get_subdirs_from_dir(path_found)
    for pid in pids:
        pids_refresh[pid.replace('_', '/')] = ','.join(ciscosupport.get_ids_from_dir(path_found+pid, refresh_time=refresh_found))

    # move not_found bug reports older then 'refresh_notfound' days to request (for refresh)
    ciscosupport.move_dir(path_not_found, path_request, refresh_time=refresh_notfound)

    # get list of PIDs requests
    pids = ciscosupport.get_subdirs_from_dir(path_request)
    logger.debug('bug requests (PIDs): %s' % pids)
    for pid in pids:
        pids_requested[pid.replace('_', '/')] = ','.join(ciscosupport.get_ids_from_dir(path_request+pid))
    logger.debug('bug requests (PIDs and releases): %s' % pids_requested)

    if len(pids_refresh.keys()) > 0 or len(pids_requested.keys()) > 0:

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=1')
#        reqoptions.append('modified_date=3')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        # reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        # wait random time after startup (load spread)
        if wait_after_start:
            ciscosupport.sleep_random(max_wait_time)

        # request access_token if no token available or if is expired
        if access_token == '':
            access_token, lifetime = get_new_token()

        if access_token == '':
            logger.critical('failed to get access_token')
            return

        # first refresh bug reports
        for pid in pids_refresh.keys():
            if lifetime > int(time.time()):
                access_token, lifetime = get_new_token()
            # get bug records for time frame
            bug_records = ciscoapi.get_bug_by_pid_and_release(pid, pids_refresh.get(pid), access_token, reqoptions)
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                logger.debug('bug return:PID: %s, Status: %s, Version: %s' % (pid, status_code, software_releases))
                path = ciscosupport.expand_path(path_found + pid.replace('/', '_'))
                # check if there where was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:  # if new/changed bugs found
                        for bug in bugs:
                            bug.pop('description')  # remove description
                        for release in software_releases.keys():  # create one bug list per software release
                            bug_release = software_releases.get(release)
                            logger.debug('bug found:PID: %s, Releases: %s' % (pid, release))
                            missing = entry.get('missing', {})
                            new_bugs = []
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    new_bugs.append(bug)

                            # open found file
                            with open(path + release) as f:
                                try:
                                    bug_record = json.load(f)
                                except ValueError, e:
                                    logging.warning('%s:%s:snmp_cisco_bug:bug_found:JSON load error: %s' % (pid, release, e))

                            found_bugs = bug_record.get('bugs')
                            for found_bug in found_bugs:
                                for new_bug in new_bugs:
                                    if found_bug.get('bug_id') == new_bug.get('bug_id'):
                                        found_bug.update(new_bug)  # update old bug with new data (changed bugs)
                                        new_bugs.remove(new_bug)
                            for bug in new_bugs:
                                found_bugs.append(bug)  # add new bugs
                            bug_record['bugs'] = found_bugs  # replace bug list with new list
                            bug_record['total_records'] = len(found_bugs)  # change number of records

                            # bug_record['missing'] = missing  # replace missing state with new missing state

                            with open(path + release, 'w') as f:
                                json.dump(bug_record, f)  # write updated bug report

                            # if len(missing.keys()) != 0:
                            #     path = ciscosupport.expand_path(path_missing + pid.replace('/', '_'))
                            #     with open(path + release, 'w') as f:
                            #         json.dump(missing, f)  # write update missing state

                    else:
                        for release in software_releases.keys():
                            if os.path.exists(path + release):
                                os.utime(path + release, None)

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=5')
#        reqoptions.append('modified_date=2')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        #reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        # pids_requested = {
        #     # 'ISR4431/K9': '03.16.05.S,03.16.04b.S',  # 03.16.05.S,03.16.04b.S
        #     # 'ISR-AP1100AC-E': '8.5.110.0',
        #     #
        #     # 'WS-X4013+TS': '12.2(25)SG4',  # --> not in database
        #     # 'WS-C3750G-24TS-1U': '12.2(55)SE8',  # WS-C3750G-24TS-E1U --> wrong PID from inventory??
        #     #
        #     # 'C1861W-UC-2BRI-K9': '15.1(4)M12a',  # pid not in database (only 1861 and 1861E not W...)
        #     # 'C6800IA-48TD': '15.0(2)EX7',
        #     # 'C6880-X-LE-16P10G': '15.1(2)SY4a',
        #     # 'C6880-X-LE-SUP': '15.1(2)SY4a',
        #     # 'CISCO2801': '15.1(4)M12a',
        #     # 'CISCO891-K9': '15.3(3)M3,15.3(3)M2',
        #     # 'WS-C2960S-F24TS-L': '15.0(2)SE10a,15.0(2)SE6',
        #     # 'WS-C2960S-F48TS-L': '15.0(2)SE10a,15.0(2)SE11,15.0(2)SE6',
        #     # 'CISCO888-K9': '15.1(4)M4,15.1(4)M5,15.3(3)M4',
        #     # 'WS-C2960G-24TC-L': '12.2(55)SE1,12.2(44)SE6',
        #     # 'CISCO881-K9': '15.2(4)M6a,15.4(3)M6a',
        #     # 'CISCO871-K9': '12.4(24)T8,12.4(15)T12,15.1(4)M8',
        #     # 'WS-C2960PD-8TT-L': '15.0(2)SE6,12.2(55)SE12,15.0(2)SE10a',
        #     #
        #     # 'N3K-C3064PQ-10GX': '7.0(3)I4(4)',
        #     # 'N5K-C5548UP': '7.0(8)N1(1)',
        #     # 'N5K-C5596UP': '7.3(2)N1(1)',
        #     # 'N5K-C56128P': '7.3(0)N1(1)',
        #     # 'N5K-C5672UP': '7.0(8)N1(1)',
        #     #
        #     # 'CISCO3845-MB': '12.4(15)T13',
        #     # 'WS-C2960-24TC-L': '12.2(44)SE6',
        #     # 'WS-C3750-24PS-S': '12.2(35)SE5',
        #     # 'WS-C3750G-12S-E': '12.2(55)SE8',
        #     # 'WS-C3750G-24PS-S': '12.2(55)SE1',
        #     #
        #     # 'N9K-C93180YC-EX': '13.2(2o)',
        #     # 'N9K-C9336PQ': '13.2(2o)',
        #     #
        #     # 'APIC-SERVER-M2': '3.2(2o)',
        #     # 'C891F-K9': '15.3(3)M4',
        #     # 'CISCO1941/K9': '15.0(1)M7',
        #     # 'CISCO876W-G-E-K9': '15.1(4)M10',
        #     # 'CISCO881-SEC-K9': '15.2(4)M6a',
        #     # 'CISCO892-K9': '15.1(4)M5',
        #     # 'WS-C2960X-24TD-L': '15.2(2)E3',
        #     # 'WS-C2960X-24TS-LL': '15.2(2)E5',
        #     # 'WS-C2960X-48FPD-L': '15.0(2)EX5',
        #     # 'WS-C2960X-48FPS-L': '15.0(2)EX5',
        #     # 'WS-C2960X-48LPD-L': '15.2(2)E3',
        #     # 'WS-C2960X-48TS-LL': '15.2(6)E',
        #     # 'WS-C3560CG-8TC-S': '15.2(2)E8',
        #     # 'WS-C3750G-48TS-S': '15.0(2)SE6',
        #     # 'WS-SUP720-3B': '15.1(2)SY2',
        #     #
        #     # 'WS-C3850-12S': '03.03.04SE',  # 3.3(4)SE
        #     # 'WS-C3650-24TS-S': '03.07.02E',  # 3.7(2)E
        #     # 'WS-C3650-48TQ': '03.03.05SE',  # 3.3(5)SE
        #     #
        #     # 'C9500-40X': '16.8.1a',
        #     # 'C1117-4PMLTEEAWE': '16.09.01',
        #     # 'ISR4351/K9': '16.09.01',
        #     # 'ASR1001-X': '16.05.01b',
        #     # 'ISR4451-X/K9': '16.05.01b',
        #     # 'C9300-48T': '16.06.03',
        #     #
        #     # 'ASA5505': '8.4(4)1',
        #     # 'ASA5506W': '9.9(2)1',
        #     # 'ASA5508': '9.6(1)5,9.6(1)10',
        #     # 'ASA5510': '9.1(6)1,8.4(7)30,9.1(7)15',
        #     'ASA5512': '9.6(1),9.2(2)4,9.6(3)8',
        #     # 'ASA5515': '9.8(1)5,9.9(2)14',
        #     # 'ASA5520': '9.1(6)11',
        #     # 'ASA5525': '9.8(1)',
        #     # 'ASA5550': '9.1(5)',
        #     # 'ASA5555': '9.9(1)2,9.7(1)4',
        #     # 'PIX-515E': '8.0(4)32',  # pix is not in database
        #     #
        #     # 'AIR-CT5520-K9': '8.0(152.0),8.2(166.0),8.2(100.0)',
        #     # 'AIR-AP1852E-E-K9': '8.2.100.0',
        #     # 'AIR-CT2504-K9': '8.5.120.0',
        # }



        #
        # try to get product series and get bugs by product series --> does not realy work :-(
        #
        #  product_series_mdf = ciscoapi.get_product_mdf_information_by_pid(pids_requested.keys(), access_token)
        # pid_product_series = {}
        # for entry in product_series_mdf:
        #     pid_product_series.update({entry.get('product_id'):entry.get('product_series')})
        #
        # product_series_release = {}
        # for pid in pid_product_series.keys():
        #     if pid_product_series.get(pid) != '':
        #         if product_series_release.get(pid_product_series.get(pid), '') == '':
        #             product_series_release.update({pid_product_series.get(pid): pids_requested.get(pid)})
        #         else:
        #             release = product_series_release.get(pid_product_series.get(pid)) + ',' + pids_requested.get(pid)
        #             product_series_release.update({pid_product_series.get(pid): release})
        #
        # for product_series in product_series_release.keys():
        #     release = product_series_release.get(product_series)
        #     release = release.split(',')
        #     release_clean = []
        #     for entry in release:
        #         if entry not in release_clean:
        #             release_clean.append(entry)
        #     product_series_release.update({product_series: ','.join(release_clean)})

        # print 'get bugs by product series and release'
        # bugs = ciscoapi.get_bug_by_productseries_and_affected_release(product_series_release, access_token, reqoptions)
        #
        # end product series
        #

        # print 'get bugs by PID and release'

        for pid in pids_requested.keys():
            if lifetime > int(time.time()):
                access_token, lifetime = get_new_token()
            bug_records = ciscoapi.get_bug_by_pid_and_release(pid, pids_requested.get(pid), access_token, reqoptions)
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                logger.debug('bug return:PID: %s, Status: %s, Version: %s' % (pid, status_code, software_releases))
                # check if there where was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:
                        for bug in bugs:
                            bug.pop('description')
                    for release in software_releases.keys():  # create one bug list per software release
                        bug_release = software_releases.get(release)
                        if bugs:
                            logger.debug('bug found:PID: %s, Releases: %s' % (pid, release))
                            missing = entry.get('missing', {})
                            # split bugs by release
                            release_bugs = {'pid': pid,
                                            'software_release': release,
                                            'bugs': [],
                                            'missing': missing}
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    release_bugs['bugs'].append(bug)
                            # changed from entry.get('total_records') --> total records for all requested software releases
                            release_bugs['total_records'] = len(release_bugs['bugs'])

                            path = ciscosupport.expand_path(path_found + pid.replace('/', '_'))
                            with open(path + release, 'w') as f:
                                json.dump(release_bugs, f)

                            if len(missing.keys()) != 0:
                                path = ciscosupport.expand_path(path_missing + pid.replace('/', '_'))
                                with open(path + release, 'w') as f:
                                    json.dump(missing, f)
                        else:
                            logger.debug('bug not found:PID: %s, Version: %s' % (pid, release))
                            path = ciscosupport.expand_path(path_not_found + pid.replace('/', '_'))
                            logger.debug('not found: %s' % entry)
                            with open(path + release, 'w') as f:
                                json.dump(entry, f)
                                pass

                        # remove request file
                        try:
                            logger.debug('bug delete request:PID: %s, Version: %s' % (pid, release))
                            os.remove(path_request + pid.replace('/', '_') + '/' + release)
                        except OSError:
                            pass

        # clean up (remove empty directories)
        ciscosupport.remove_empty_sub_dirs(path_request)
        ciscosupport.remove_empty_sub_dirs(path_found)
        ciscosupport.remove_empty_sub_dirs(path_not_found)
        ciscosupport.remove_empty_sub_dirs(path_missing)

main()



